/**
 * \file
 * \brief !BETA! Example of creating threads for the EDF scheduler
 *
 * This example shows how threads owning a deadline are created for the EDF scheduler.
 *
 * \date   Aug 27th 2014
 * \author Valentin Hauner <vh@hnr.name>
 */
/* 
 * (c) 2014 Valentin Hauner <vh@hnr.name>
 *
 * This file is distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 */
#include <l4/libedft/edft.h>

#include <stdio.h>
#include <unistd.h>

void thread_func(l4_umword_t);

char *thread_color[] = { ANSI_COLOR_BLUE, ANSI_COLOR_GREEN, ANSI_COLOR_MAGENTA, ANSI_COLOR_YELLOW };

void thread_func(l4_umword_t no)
{
  unsigned i, k;

  printf(thread_color[no]);
  k = edft_obj(no)->wcet;
  for (i = 1; i <= k; i++)
    printf("Thread %lu: Hello World %d!\n", no, i);
  printf(ANSI_RESET "\n");

  edft_release_cap(no);
}

int main(void)
{
  l4_debugger_set_object_name(l4re_env()->main_thread, "main_thread");

  // Initialization: no - name - wcet - dl - func - arg - suc - cap
  Edf_thread t0 = { 0, "thread0",  5, -1, thread_func, 0, NULL, 0 };
  Edf_thread t1 = { 1, "thread1", 10, -1, thread_func, 1, NULL, 0 };
  Edf_thread t2 = { 2, "thread2", 15, -1, thread_func, 2, NULL, 0 };
  Edf_thread t3 = { 3, "thread3", 20, -1, thread_func, 3, NULL, 0 };

  // Set up precedence relations
  Edf_suc suc01 = { &t2, NULL };
  Edf_suc suc0  = { &t1, &suc01 };
  Edf_suc suc1  = { &t3, NULL };
  t0.suc = &suc0;
  t1.suc = &suc1;

  // Create threads according to their precedence relations
  edft_create_dependent_threads(&t0, 0, 1);

  edft_print_precedence_graph(&t0, 0);

  //enter_kdebug();

  /*
  // Create threads manually:
  create_l4_edf_thread(t0);
  create_l4_edf_thread(t1);
  create_l4_edf_thread(t2);
  create_l4_edf_thread(t3);
  */

  /*
  // Idle around...
  while (1)
    sleep(1);
  */
}
